/*
  --------------------------------------------------------------------------------------------------------------------
   Example sketch/program showing An Arduino Door Access Control featuring RFID, EEPROM, Relay
  ---------------------------------------------------------------------------------------------------------------------

  This example showing a complete Door Access Control System

  Simple Work Flow (not limited to) :
                                 +--------------------+
  +------------------------------>      READ TAGS     <-------------------------------------+
  |                              +--------------------+                                     |
  |                              |                    |                                     |
  |                         +----v-----+        +-----v----+                                |
  |                         |MASTER TAG|        |OTHER TAGS|                                |
  |                         +--+-------+        ++-------------+                            |
  |                            |                 |             |                            |
  |                      +-----v---+        +----v----+   +----v------+                     |
  |         +------------+READ TAGS+---+    |KNOWN TAG|   |UNKNOWN TAG|                     |
  |         |            +-+-------+   |    +-----------+ +------------------+              |
  |         |              |           |                |                    |              |
  |    +----v-----+   +----v----+   +--v--------+     +-v----------+  +------v----+         |
  |    |MASTER TAG|   |KNOWN TAG|   |UNKNOWN TAG|     |GRANT ACCESS|  |DENY ACCESS|         |
  |    +-----+----+   +---+-----+   +-----+-----+     +-----+------+  +-----+-----+         |
  |          |            |               |                 |               |               |
  |       +--v--+    +----v------+     +--v---+             |               +--------------->
  +-------+EXIT |    |DELETE FROM|     |ADD TO|             |                               |
          +-----+    |  DATABASE |     |  DB  |             |                               |
                     +-----------+     +------+             +-------------------------------+

   Use a Master Card which is act as Programmer then you can able to choose card holders who will granted access or not

 * **Easy User Interface**

   Just one RFID tag needed whether Delete or Add Tags. You can choose to use Leds for output or Serial LCD module to inform users.

 * **Stores Information on EEPROM**

   Information stored on non volatile Arduino's EEPROM memory to preserve Users' tag and Master Card. No Information lost
   if power lost. EEPROM has unlimited Read cycle but roughly 100,000 limited Write cycle.

    Typical pin layout used:
   -----------------------------------------------------------------------------------------
               MFRC522      Arduino       Arduino   Arduino    Arduino          Arduino
               Reader/PCD   Uno/101       Mega      Nano v3    Leonardo/Micro   Pro Micro
   Signal      Pin          Pin           Pin       Pin        Pin              Pin
   -----------------------------------------------------------------------------------------
   RST/Reset   RST          9             5         D9         RESET/ICSP-5     RST
   SPI SS      SDA(SS)      10            53        D10        10               10
   SPI MOSI    MOSI         11 / ICSP-4   51        D11        ICSP-4           16
   SPI MISO    MISO         12 / ICSP-1   50        D12        ICSP-1           14
   SPI SCK     SCK          13 / ICSP-3   52        D13        ICSP-3           15

*/

#include <EEPROM.h>    // We are going to read and write cards IDs from/to EEPROM
#include <SPI.h>       // RC522 Module uses SPI protocol
#include <MFRC522.h>   // Library for Mifare RC522 Devices

#define SS_PIN 10
#define RST_PIN A5
MFRC522 mfrc522(SS_PIN, RST_PIN);   // Create MFRC522 instance.

#define RedLED  6
#define YellowLED  5
#define BlueLED  3

#define Relay  A0
#define Taster 4
#define Buzzer 2
#define wipeB 4     // Button pin for WipeMode
boolean wipeTaster, pwipeTaster;

#define NO_ACTION         0
#define ACCESS_GRANTED    1
#define ACCESS_DENIDED    2
#define PROGRAM_MODE      3
#define CARD_DELETED      4
#define CARD_ADDED        5
#define NEW_MASTER        6

#define MIN_PAUZA     300
#define SIG_DURATION    3000
#define SET_LED_TIME    250

#include "pitches.h"

////////////////////Podaci i promjenljive za vezu sa bazom podataka
#include <SoftwareSerial.h>
#include <avr/pgmspace.h>

// include the library code FOR i2c LCD:
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

// initialize the library with the numbers of the interface pins
LiquidCrystal_I2C lcd(0x27, 16, 2);

String upperRow;
String lowerRow;
boolean refreshLCD = false;
long defLCD = 0;
#define REFRESH     5000


#define RST_ESP  A1

SoftwareSerial esp(7, 8);// RX, TX

String espCommand;

String ReadTag;
int terminalID = 1;

String data;
String server = "89.188.33.127"; //"89.188.33.127"; //"192.168.100.8"; // www.example.com

String postData;
int pDataLen;

#define MAX_LENTH_OF_POST_REQUEST 512 /* Adjust the size to your needs */
String postRequest;
int pRqstLen;

// Keep this API Key value to be compatible with the PHP code provided in the project page.
// If you change the apiKeyValue value, the PHP file /post-esp-data.php also needs to have the same key
String apiKeyValue = "tPmAT5Ab3j7F9";

///////////////Kraj podataka i promjenljivih za vezu sa bazom podataka


// notes in the melody:
int AGmelody[] = {
  NOTE_C4, NOTE_G3, NOTE_G3, NOTE_A3, NOTE_G3, 0, NOTE_B3, NOTE_C4
};
int ADmelody[] = {
  NOTE_F6, NOTE_F6, NOTE_F6, NOTE_F6, NOTE_F6, NOTE_F6, NOTE_F6, NOTE_F6
};

// note durations: 4 = quarter note, 8 = eighth note, etc.:
int AGnoteDurations[] = {
  4, 8, 8, 4, 4, 4, 4, 4
};
int ADnoteDurations[] = {
  4, 4, 4, 4, 4, 4, 4, 4
};
#define FULL_MELODY   8
#define ONE_NOTE      1
unsigned char notes, currentNote;
int* melody;
int* noteDur;

boolean newCard;
int SigMod;
long detectionTime;
long setLED;

uint8_t successRead;    // Variable integer to keep if we have Successful Read from Reader

byte storedCard[8];   // Stores an ID read from EEPROM
byte readCard[8];   // Stores scanned ID read from RFID Module
byte masterCard[8]; //{0x3B, 0xD2, 0x2B, 0x1B}; // Master card's ID

boolean newMaster = false;

bool programMode = false;  // initialize programming mode

//Melody function
void Sound(int* melody, int* noteDurations, unsigned char curNote) {
  // To calculate the note duration, take one second divided by the note type.
  //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
  int noteDuration = 1000 / noteDurations[curNote];
  tone(Buzzer, melody[curNote], noteDuration);
  delay(noteDuration);
}

//Light and sound signalisation
void Signals(int Mod)
{
  static int noAct;

  if (Mod == NO_ACTION) {
    if (millis() - setLED > SET_LED_TIME)
    {
      if (noAct % 5) {
        digitalWrite(RedLED, LOW);
        digitalWrite(BlueLED, LOW);
        digitalWrite(YellowLED, LOW);
      }
      else if (noAct % 5 == 0) {
        digitalWrite(RedLED, LOW);
        digitalWrite(BlueLED, LOW);
        digitalWrite(YellowLED, HIGH);
      }
      noAct++;
      setLED = millis();
      digitalWrite(Relay, LOW);
      noTone(Buzzer);
    }
  }
  else if (Mod == PROGRAM_MODE) {
    if (millis() - setLED > SET_LED_TIME)
    {
      if (!(noAct % 5)) {
        digitalWrite(RedLED, LOW);
        digitalWrite(BlueLED, LOW);
        digitalWrite(YellowLED, LOW);
      }
      else if (noAct % 5) {
        digitalWrite(RedLED, HIGH);
        digitalWrite(BlueLED, HIGH);
        digitalWrite(YellowLED, HIGH);
      }
      noAct++;
      setLED = millis();
      digitalWrite(Relay, LOW);
      noTone(Buzzer);
    }
  }
  else if (Mod == ACCESS_GRANTED) {
    digitalWrite(RedLED, LOW);
    digitalWrite(BlueLED, HIGH);
    digitalWrite(YellowLED, LOW);
    digitalWrite(Relay, HIGH);
  }
  else if (Mod == ACCESS_DENIDED) {
    digitalWrite(RedLED, HIGH);
    digitalWrite(BlueLED, LOW);
    digitalWrite(YellowLED, LOW);
    digitalWrite(Relay, LOW);
  }
  else if (Mod == CARD_ADDED) {
    digitalWrite(RedLED, LOW);
    digitalWrite(BlueLED, HIGH);
    digitalWrite(YellowLED, HIGH);
    digitalWrite(Relay, LOW);
  }
  else if (Mod == CARD_DELETED) {
    digitalWrite(RedLED, HIGH);
    digitalWrite(BlueLED, LOW);
    digitalWrite(YellowLED, HIGH);
    digitalWrite(Relay, LOW);
  }
  else if (Mod == NEW_MASTER) {
    digitalWrite(RedLED, HIGH);
    digitalWrite(BlueLED, HIGH);
    digitalWrite(YellowLED, HIGH);
    digitalWrite(Relay, LOW);
  }
}


//Compare to 4 bytes array
bool checkTwo ( byte *a, byte *b ) {
  for ( uint8_t k = 0; k < 4; k++ ) {   // Loop 4 times
    if ( a[k] != b[k] ) {     // IF a != b then false, because: one fails, all fail
      return false;
    }
  }
  return true;
}


//Read card ID
uint8_t ReadCardID(byte* readTag) {
  if ( ! mfrc522.PICC_ReadCardSerial()) { //Since a PICC placed get Serial and continue
    return 0;
  }
  // We support 4 byte card ID
  Serial.println(F("Scanned card ID:"));
  ReadTag = "";
  for ( uint8_t i = 0; i < 4; i++) {  //
    readTag[i] = mfrc522.uid.uidByte[i];
    ReadTag.concat(String(readTag[i] < 0x10 ? "0" : ""));
    ReadTag.concat(String(readTag[i], HEX));
  }
  Serial.print(ReadTag);
  readTag[4] = '\0';
  Serial.println("");
  mfrc522.PICC_HaltA(); // Stop reading
  return 1;
}


// Check to see if the ID passed is the master programing card
bool isMaster( byte test[] ) {
  return checkTwo(test, masterCard);
}


// Read an ID from EEPROM
void readID( uint8_t number, byte sCard[]) {
  uint8_t start = (number * 4 ) + 6; // Figure out starting position
  for ( uint8_t i = 0; i < 4; i++ ) { // Loop 4 times to get the 4 Bytes
    sCard[i] = EEPROM.read(start + i); // Assign values read from EEPROM to array
    delay(1);
  }
}

// Writing New Master ID to EEPROM
byte writeNewMasterID(char* newMasterCard) {
  uint8_t start = 2;  // Figure out where the next slot starts
  for ( uint8_t j = 0; j < 4; j++ ) { // Loop 4 times
    EEPROM.write( start + j, newMasterCard[j] ); // Write the array values to EEPROM in the right position
  }
  Serial.println(F("Succesfully writed New Master ID record to EEPROM"));
  return (1);
}


// Reading Master ID from EEPROM
void readMasterID(byte* masterTag) {
  uint8_t start = 2; // Figure out starting position
  for ( uint8_t i = 0; i < 4; i++ ) { // Loop 4 times to get the 4 Bytes
    masterTag[i] = EEPROM.read(start + i); // Assign values read from EEPROM to array
    Serial.print(masterTag[i], HEX);
    delay(1);
  }
  Serial.println();
  Serial.println(F("Succesfully read Master ID record from EEPROM"));
}

void writeToLCD(String upRow, String lwRow) {
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(upRow);
  lcd.setCursor(0, 1);
  lcd.print(lwRow);
  Serial.println(upRow);
  Serial.println(lwRow);
}


///////////////////////// Funkcije za rad sa esp modulom i komunikacijom sa bazom podataka

#define ATTEMPTS 3
int responseTime = 10000; //communication timeout
long WAKEUP_PERIOD = 10000;
#define REQUEST_LENGTH   154;
bool DEBUG = true;   //show more logs
String responses; //String u koji se smjesta odgovor modula

void(* resetFunc) (void) = 0;//declare reset function at address 0


/*
  Name: find
  Description: Function used to match two string
  Params:
  Returns: true if match else false
*/
boolean find(String string, String value) {
  if (string.indexOf(value) >= 0)
    return true;
  return false;
}

/*
  Name: sendToESP8266
  Description: Function used to send data to ESP8266.
  Params: command - the data/command to send; timeout - the time to wait for a response; debug - print to Serial window?(true = yes, false = no)
  Returns: The response from the esp8266 (if there is a reponse)
*/
int sendToESP8266(String command, const int timeout, boolean debug, String rsp, String response) {
  response = "";
  int rez = 1;
  int atmp = 0;
  while (atmp < ATTEMPTS)
  {
    if (debug)Serial.println(command);
    esp.println(command); // send the read character to the SIM900
    long int time = millis();
    while ( (time + timeout) > millis())
    {
      if (esp.available())
      {
        // The esp has data so display its output to the serial window
        char c = esp.read(); // read the next character.
        response += c;
      }
      if (rsp != "")
      {
        if (find(response, rsp)) {
          rez = 0;
          break;
        }
      }
      else {
        rez = 0;
        break;
      }
    }
    if (debug)
    {
      Serial.println(response); Serial.println(rez);
    }
    if (!rez)break;
    atmp++;
  }
  return rez;
}


//reset the esp8266 module
void resetESP() {
  //Reset ESP module
  digitalWrite(RST_ESP, LOW);
  delay(500);
  digitalWrite(RST_ESP, HIGH);
  if (sendToESP8266("", responseTime, DEBUG, F("ready"), responses) != 0) {
    resetFunc();
  }

  if (sendToESP8266(F("AT"), responseTime, DEBUG, F("OK"), responses) != 0) {
    resetFunc();
  }

  Serial.println(F("Module Reset"));
}

//Set up as station
void SetUpESP() {
  //Podešavanje ESP da radi kao klijent
  String cmd = (String)(F("AT+CWMODE=1"));
  if (sendToESP8266(cmd, responseTime, DEBUG, "OK", responses) != 0) {
    resetFunc();
  }
  if (sendToESP8266(F("AT+CIPMUX=0"), responseTime, DEBUG, "OK", responses) != 0) {
    resetFunc();
  }

  //Povezivanje na WiFi mrežu
  cmd = (String)(F("AT+CWJAP_CUR=\"")) + F("Lovcen") + F("\",\"") + F("") + F("\"");
  if (sendToESP8266(cmd, responseTime, DEBUG, "OK", responses) != 0) {
    Serial.println(F("Cannot connect to wifi"));
    resetFunc();
  }
  Serial.println(F("Connected!"));
}


// HTTP POST request sending
int httppost(String Uri, String SerVer) {
  int fResponse;

  //TCP conncetion establishing
  espCommand = String(F("AT+CIPSTART=\"TCP\",\"")) + SerVer + F("\",80");
  Serial.println(espCommand);
  if (sendToESP8266(espCommand, responseTime, DEBUG, "OK", responses) != 0) {
    resetFunc();
  }
  Serial.println(F("TCP connection ready"));

  //Preparing to send a POST request
  pDataLen = postData.length();
  Serial.println(pDataLen);

  postRequest = "";
  postRequest = String(F("POST ")) + Uri + F(" HTTP/1.1\r\n") + F("Host: ") + SerVer + F("\r\n");
  postRequest += String(F("Accept: */*\r\nContent-Length: ")) + String(pDataLen) + F("\r\n");
  postRequest += String(F("Content-Type: application/x-www-form-urlencoded\r\n\r\n"));

  Serial.print(postRequest);
  Serial.println(postData);

  pRqstLen = postRequest.length() + pDataLen;
  Serial.println(pRqstLen);

  //POST request sending
  espCommand = String(F("AT+CIPSEND=")) + String(pRqstLen);
  if (sendToESP8266(espCommand, responseTime, DEBUG, ">", responses) != 0) {
    resetFunc();
  }
  Serial.println(F("Sending.."));
  esp.print(postRequest);
  esp.print(postData);
  if (sendToESP8266("", responseTime, DEBUG, "SEND OK", responses) != 0) {
    resetFunc();
  }
  Serial.println(F("POST Request Sent"));

  //Response reading and analyzing
  String tmpResp = esp.readString();
  Serial.println(tmpResp);
  if (tmpResp.indexOf(F("accepted")) != (-1)) {
    upperRow = F("Access granted!");
    lowerRow = F("WELCOME");
    refreshLCD = true; defLCD = millis();
    notes = FULL_MELODY; melody = AGmelody, noteDur = AGnoteDurations; currentNote = 0;
    fResponse = ACCESS_GRANTED;
  }
  else if (tmpResp.indexOf(F("denided")) != (-1)) {
    upperRow = F("Access denided!");
    lowerRow = F("SORRY!");
    refreshLCD = true; defLCD = millis();
    fResponse = ACCESS_DENIDED;
    notes = FULL_MELODY; melody = ADmelody, noteDur = ADnoteDurations; currentNote = 0;
  }
  else if (tmpResp.indexOf(F("deleted")) != (-1)) {
    upperRow = F("Card");
    lowerRow = F("deleted");
    refreshLCD = true; defLCD = millis();
    notes = FULL_MELODY; melody = ADmelody, noteDur = ADnoteDurations; currentNote = 0;
    fResponse = CARD_DELETED;
  }
  else if (tmpResp.indexOf(F("added")) != (-1)) {
    upperRow = F("Card");
    lowerRow = F("added");
    refreshLCD = true; defLCD = millis();
    notes = FULL_MELODY; melody = AGmelody, noteDur = AGnoteDurations; currentNote = 0;
    fResponse = CARD_ADDED;
  }
  else if (tmpResp.indexOf(F("Row created")) != (-1)) {
    upperRow = F("Row created");
    lowerRow = F("succ...");
    refreshLCD = true; defLCD = millis();
    notes = FULL_MELODY; melody = AGmelody, noteDur = AGnoteDurations; currentNote = 0;
    fResponse = CARD_ADDED;
  }
  else if (tmpResp.indexOf(F("Error")) != (-1)) {
    upperRow = F("ERROR!!!");
    lowerRow = F("ERROR!!!");
    refreshLCD = true; defLCD = millis();
    notes = FULL_MELODY; melody = ADmelody, noteDur = ADnoteDurations; currentNote = 0;
    fResponse = ACCESS_DENIDED;
  }

  //Closing the TCP connection
  if (sendToESP8266(F("AT+CIPCLOSE"), responseTime, DEBUG, F("CLOSED"), responses) != 0) {
    resetFunc();
  }

  return fResponse;
}
///////////////////////// Kraj funkcija za rad sa esp modulom i komunikacijom sa bazom podataka


void setup()
{
  //Konfigurisanje pinova
  pinMode(RedLED, OUTPUT);
  pinMode(YellowLED, OUTPUT);
  pinMode(BlueLED, OUTPUT);
  pinMode(Relay, OUTPUT);
  pinMode(Taster, INPUT_PULLUP);
  pinMode(Buzzer, OUTPUT);
  pinMode(RST_ESP, OUTPUT);
  digitalWrite(RST_ESP, HIGH);

  Serial.begin(115200);   // Initiate a serial communication
  Serial.setTimeout(100);
  SPI.begin();      // Initiate  SPI bus
  mfrc522.PCD_Init();   // Initiate MFRC522

  esp.begin(9600);  // Initiate a sofrware serial communication
  esp.setTimeout(100);
  resetESP(); // Reset the esp module
  SetUpESP(); // Setup esp module as station

  // Read the Master tag(card) ID from EEPROM
  Serial.println();
  readMasterID(masterCard);
  Serial.println();

  //Inicijalizacija nacina signalizacije
  SigMod = NO_ACTION;
  newCard = true;

  //Wipe taster chack - If the taster (wipeB) pressed longer than 10s it wipes EEPROM
  if (digitalRead(wipeB) == LOW) {  // when button pressed pin should get low, button connected to ground
    newMaster = true;
    if (newMaster)SigMod = NEW_MASTER;
    Serial.println(F("New MASTER mode started!!"));
    Serial.println(F("-----------------------------"));
  }

  wipeTaster = digitalRead(wipeB);
  pwipeTaster = wipeTaster;

  Serial.println(F("Približite identifikator čitaču..."));
  Serial.println();

  // Inicijalizacija LCD-a
  lcd.init();
  // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  //To power on backlight
  lcd.backlight();

  upperRow = String(F("Identify mode!"));
  lowerRow = String(F("WELCOME!"));
  refreshLCD = true;
}


//Main loop
void loop()
{
  // Look for new cards
  if ( mfrc522.PICC_IsNewCardPresent())
  {
    detectionTime = millis();
    successRead = ReadCardID(readCard);
    if (!successRead)return;
    if (newMaster) {
      Serial.println(F("Writing new Master ID"));
      Serial.println(F("-----------------------------"));
      writeNewMasterID(readCard);
      //Read the Master tag(card) ID from EEPROM
      readMasterID(masterCard);
      newMaster = false;
    }
    else if (programMode) {
      SigMod = PROGRAM_MODE;
      if ( isMaster(readCard) ) { //When in program mode check First If master card scanned again to exit program mode
        Serial.println(F("Master Card Scanned"));
        Serial.println(F("Exiting Program Mode"));
        Serial.println(F("-----------------------------"));

        upperRow = F("Master again");
        lowerRow = F("Identify mode!");
        refreshLCD = true; defLCD = millis();

        programMode = false;
        SigMod = NO_ACTION;
      }
      else {
        // data sent must be under this form //name1=value1&name2=value2.
        postData = String(F("api_key=")) + apiKeyValue + F("&terminalID=") + terminalID;
        postData += String(F("&tagID=")) + ReadTag + "";

        upperRow = F("Card Scanned");
        lowerRow = F("Ins/del user...");
        writeToLCD(upperRow, lowerRow);

        SigMod = httppost(F("/idsysev/post_insdel_user.php"), server);
        detectionTime = millis();
      }
    }
    else {
      if ( isMaster(readCard)) {    // If scanned card's ID matches Master Card's ID - enter program mode
        programMode = true;
        Serial.println(F("Hello Master - Entered Program Mode"));
        uint8_t count = EEPROM.read(0);   // Read the first Byte of EEPROM that
        Serial.print(F("I have "));     // stores the number of ID's in EEPROM
        Serial.print(count);
        Serial.print(F(" record(s) on EEPROM"));
        Serial.println("");
        Serial.println(F("Scan a tag(card) to ADD or REMOVE to EEPROM"));
        Serial.println(F("Scan Master tag(card) again to Exit Program Mode"));
        Serial.println(F("-----------------------------"));

        upperRow = F("Master Scanned");
        lowerRow = F("Program mode!");
        refreshLCD = true; defLCD = millis();

        SigMod = PROGRAM_MODE;
      }
      else {
        // data sent must be under this form //name1=value1&name2=value2.
        postData = String(F("api_key=")) + apiKeyValue + F("&terminalID=") + terminalID;
        postData += String(F("&tagID=")) + ReadTag + "";

        upperRow = F("Card Scanned");
        lowerRow = F("Check in prog...");
        writeToLCD(upperRow, lowerRow);

        SigMod = httppost(F("/idsysev/post_evid_data.php"), server);
        detectionTime = millis();
      }
    }
    mfrc522.PICC_HaltA ();
  }

  //Wipe taster chack - If the taster (wipeB) pressed longer than 10s it wipes EEPROM
  wipeTaster = digitalRead(wipeB);
  if ((wipeTaster == LOW) && (wipeTaster != pwipeTaster)) {
    newMaster = !newMaster;
    if (newMaster) {
      Serial.println(F("New MASTER mode started!!"));
      Serial.println(F("-----------------------------"));

      upperRow = F("Master mode");
      lowerRow = F("Waiting card...");
      refreshLCD = true; defLCD = millis();

      SigMod = NEW_MASTER;
    }
  }
  pwipeTaster = wipeTaster;

  //Signalisation
  if ( millis() - detectionTime > SIG_DURATION) {
    if (newMaster)SigMod = NEW_MASTER;
    else if (programMode)SigMod = PROGRAM_MODE;
    else SigMod = NO_ACTION;
  }
  Signals(SigMod);

  //Provjera treba li vratiti default ispis
  if ( ((millis() - defLCD) > REFRESH) && defLCD)  {
    if (programMode)upperRow = String(F("Program mode!"));
    else upperRow = String(F("Identify mode!"));
    lowerRow = String(F("WELCOME!"));
    defLCD = 0;
    refreshLCD = true;
  }

  //Ispis na LCD-u
  if (refreshLCD) {
    writeToLCD(upperRow, lowerRow);
    refreshLCD = false;
  }

  //Zvučni signal
  if (currentNote < notes)
  {
    Sound(melody, noteDur, currentNote);
    currentNote++;
  }
  else noTone(Buzzer);
}
